use std::error::Error;

use crossterm::event::{self, KeyCode};

use crate::{
    state::{AppMode, ApplicationState, Task},
    utility::Op,
};

/// Handle all key events.
/// This function maps any user key input to the corresponding application state update
pub fn event_key_handler(app: &mut ApplicationState) -> Result<(), Box<dyn Error>> {
    if event::poll(std::time::Duration::from_millis(100))? {
        if let event::Event::Key(key_event) = event::read()? {
            match app.mode {
                AppMode::Board => board_key_handler(app, &key_event.code)?,
                AppMode::NewTask => panic!("Not implemented yet"),
            }
        }
    };
    Ok(())
}

fn board_key_handler(app: &mut ApplicationState, code: &KeyCode) -> Result<(), Box<dyn Error>> {
    match code {
        KeyCode::Esc | KeyCode::Char('q') => app.should_quit = true,
        KeyCode::Char('a') => {
            app.popup.clear();
            app.mode = AppMode::NewTask
        }
        KeyCode::Char('h') => app.update_selected_column(&Op::Decrement),
        KeyCode::Char('l') => app.update_selected_column(&Op::Increment),
        KeyCode::Char('k') => app.update_selected_item(&Op::Decrement),
        KeyCode::Char('j') => app.update_selected_item(&Op::Increment),
        KeyCode::Char('H') => {
            app.move_task(app.selected_col, app.selected_item, &Op::Decrement, false)
        }
        KeyCode::Char('L') => {
            app.move_task(app.selected_col, app.selected_item, &Op::Increment, false)
        }
        KeyCode::Char('K') => {
            app.move_task(app.selected_col, app.selected_item, &Op::Decrement, true)
        }
        KeyCode::Char('J') => {
            app.move_task(app.selected_col, app.selected_item, &Op::Increment, true)
        }
        KeyCode::Char('x') => app.remove_task(app.selected_col, app.selected_item),
        _ => {}
    }
    Ok(())
}
